///////////////////////////////////////////////////////////////////////////////
// Globals

const kObserverServiceProgID = "@mozilla.org/observer-service;1";
const NC_NS = "http://www.fbay.com/rdf#";
const NC_ROOT = "http://www.fbay.com/uploads";
const PREF_BDM_CLOSEWHENDONE = "browser.download.manager.closeWhenDone";
const PREF_BDM_ALERTONEXEOPEN = "browser.download.manager.alertOnEXEOpen";

var gUploadManager  = null;
var gUploadListener = null;
var gUploadsView    = null;
var gUserInterfered   = false;
var gActiveDownloads  = [];

// This variable exists because for XPInstalls, we don't want to close the 
// download manager until the XPInstallManager sends the DIALOG_CLOSE status
// message. Setting this variable to false when the downloads window is 
// opened by the xpinstall manager prevents the window from being closed after
// each download completes (because xpinstall downloads are done sequentially, 
// not concurrently) 
var gCanAutoClose     = true;

///////////////////////////////////////////////////////////////////////////////
// Utility Functions 
function setRDFProperty(aID, aProperty, aValue)
{
  var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);

  var db = gUploadManager.datasource;
  var propertyArc = rdf.GetResource (NC_NS + aProperty);
  
  var res = rdf.GetResource (aID);
  var node = db.GetTarget (res, propertyArc, true);
  if (node)
    db.Change (res, propertyArc, node, rdf.GetLiteral (aValue));
  else
    db.Assert (res, propertyArc, rdf.GetLiteral (aValue), true);
}

function getRDFProperty(aID, aProperty)
{
  var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);

  var db = gUploadManager.datasource;
  var propertyArc = rdf.GetResource(NC_NS + aProperty);
  
  var res = rdf.GetResource(aID);
  var node = db.GetTarget(res, propertyArc, true);
  if (!node) return "";
  try {
    node = node.QueryInterface(Components.interfaces.nsIRDFLiteral);
    return node.Value;
  }
  catch (e) {
    try {
      node = node.QueryInterface(Components.interfaces.nsIRDFInt);
      return node.Value;
    }
    catch (e) {
      node = node.QueryInterface(Components.interfaces.nsIRDFResource);
      return node.Value;
    }
  }
  return "";
}

function fireEventForElement(aElement, aEventType)
{
  var e = document.createEvent("Events");
  e.initEvent("download-" + aEventType, false, true);
  
  aElement.dispatchEvent(e);
}

///////////////////////////////////////////////////////////////////////////////
// Start/Stop Observers
function downloadCompleted(aDownload) 
{
  // Wrap this in try...catch since this can be called while shutting down... 
  // it doesn't really matter if it fails then since well.. we're shutting down
  // and there's no UI to update!
  try {
    var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
    var rdfc = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);

    var db = gUploadManager.datasource;
    
    rdfc.Init(db, rdf.GetResource("NC:DownloadsRoot"));

    var id = aDownload.targetFile.path;
    
    // getTypeFromFile fails if it can't find a type for this file. Handle this gracefully.
    try {
      // Refresh the icon, so that executable icons are shown.
      var mimeService = Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"].getService(Components.interfaces.nsIMIMEService);
      var contentType = mimeService.getTypeFromFile(aDownload.targetFile);

      var listItem = document.getElementById(id);
      var oldImage = listItem.getAttribute("image");
      // I tack on the content-type here as a hack to bypass the cache which seems
      // to be interfering despite the fact the image has 'validate="always"' set
      // on it. 
      listItem.setAttribute("image", oldImage + "&contentType=" + contentType);
    } catch (e) {
    }
    
    var dlRes = rdf.GetUnicodeResource(id);
  
    var insertIndex = gUploadManager.activeDownloadCount + 1;
    // Don't bother inserting the item into the same place!
    if (insertIndex != rdfc.IndexOf(dlRes)) {
      rdfc.RemoveElement(dlRes, true);
      if (insertIndex == rdfc.GetCount() || insertIndex < 1) 
        rdfc.AppendElement(dlRes);
      else
        rdfc.InsertElementAt(dlRes, insertIndex, true);      
    }
        
    // Remove the download from our book-keeping list and if the count
    // falls to zero, update the title here, since we won't be getting
    // any more progress notifications in which to do it. 
    for (var i = 0; i < gActiveDownloads.length; ++i) {
      if (gActiveDownloads[i] == aDownload) {
        gActiveDownloads.splice(i, 1);
        break;
      }
    }

    gUploadViewController.onCommandUpdate();

    if (gActiveDownloads.length == 0)
      document.title = document.documentElement.getAttribute("statictitle");
  }
  catch (e) {
  }
}

function autoClose(aDownload)
{
  if (gUploadManager.activeDownloadCount == 0) {
    // For the moment, just use the simple heuristic that if this window was
    // opened by the download process, rather than by the user, it should auto-close
    // if the pref is set that way. If the user opened it themselves, it should
    // not close until they explicitly close it. 
    // We may like to revisit this in a bit more detail later, perhaps we want
    // to keep it up if the user messes with it in a significant way.
    var pref = Components.classes["@mozilla.org/preferences-service;1"]
                        .getService(Components.interfaces.nsIPrefBranch);
    var autoClose = pref.getBoolPref(PREF_BDM_CLOSEWHENDONE)
    if (autoClose && (!window.opener || window.opener.location.href == window.location.href) &&
        gCanAutoClose)
      gCloseDownloadManager();
  }
}

// This function can be overwritten by extensions that wish to place the Download Window in
// another part of the UI, such as in a tab or a sidebar panel. 
function gCloseDownloadManager()
{
  window.close();
}

function calcHash (f)
{
  var hash = Components.classes["@fileauction.com/hash/fa;1"]
                       .createInstance(Components.interfaces.IFileAuctionHash);
  var file = Components.classes["@mozilla.org/file/local;1"]
                       .createInstance(Components.interfaces.nsILocalFile);
  file.initWithPath (f.path);

  return hash.calc (file);
}

function fileauction_UpdateDoneStatus (aDownload)
{
  var req_fsize = window.arguments[2] * 1;
  var req_fhash = window.arguments[3];

  var id = aDownload.targetFile.path;
  var f = getLocalFileFromNativePathOrUrl (id);

  if (f.exists ())
    {
      // if can't find the file.
      try {
	var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
        	           .createInstance(Components.interfaces.nsIFileInputStream);
	is.init (f, 0x01, 00004, null);
	var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
		            .createInstance(Components.interfaces.nsIScriptableInputStream);
	sis.init (is);
	var output = sis.read (256);
	sis.close ();

	if (output.match (RegExp ("^<FileAuction> Can't find the file")))
	  {
	    sendSignalToCompany (window.arguments[4], "CantFindFile");
	    setRDFProperty(id, "DownloadState", "10");
	    return;
	  }
      } catch (e) { }

      // check size
      if (req_fsize != f.fileSize)
	{
	  sendSignalToCompany (window.arguments[4], "IncorrectFileSize");
	  setRDFProperty(id, "DownloadState", "9");
	  return;
	}

      // check hash.
      setRDFProperty(id, "DownloadState", "11");
      var h = calcHash (f);
      if (h != req_fhash)
	{
	  sendSignalToCompany (window.arguments[4], "InvalidHash");
	  setRDFProperty(id, "DownloadState", "12");
	  return;
	}

      sendSignalToCompany (window.arguments[4], "DownloadFinished");
      setRDFProperty(id, "DownloadState", "1");
    }
  else
    {
      sendSignalToCompany (window.arguments[4], "CantConnectToSeller");
      setRDFProperty(id, "DownloadState", "8"); // connection error.
    }
}

var gUploadObserver = {
  observe: function (aSubject, aTopic, aState) 
  {
    switch (aTopic) {
    case "dl-done":
      var dl = aSubject.QueryInterface(Components.interfaces.nsIDownload);
      downloadCompleted(dl);
      fileauction_UpdateDoneStatus (dl);
      autoClose(dl);
      break;
    case "dl-failed":
    case "dl-cancel":
      var dl = aSubject.QueryInterface(Components.interfaces.nsIDownload);
      downloadCompleted(dl);
      break;
    case "dl-start":
      // whenever a new download starts, it is added to the top, so switch the
      // view to it
      gUploadsView.selectedIndex = 0;

      // Add this download to the percentage average tally
      var dl = aSubject.QueryInterface(Components.interfaces.nsIDownload);
      gActiveDownloads.push(dl);

      break;
    case "xpinstall-download-started":
      var windowArgs = aSubject.QueryInterface(Components.interfaces.nsISupportsArray);
      var params = windowArgs.QueryElementAt(0, Components.interfaces.nsISupportsInterfacePointer);
      params = params.data.QueryInterface(Components.interfaces.nsIDialogParamBlock);
      var installObserver = windowArgs.QueryElementAt(1, Components.interfaces.nsISupportsInterfacePointer);
      installObserver = installObserver.data.QueryInterface(Components.interfaces.nsIObserver);
      XPInstallDownloadManager.addDownloads(params, installObserver);
      break;  
    case "xpinstall-dialog-close":
      if ("gUploadManager" in window) {
        var mgr = gUploadManager.QueryInterface(Components.interfaces.nsIXPInstallManagerUI);
        gCanAutoClose = mgr.hasActiveXPIOperations;
        autoClose();
      }
      break;          
    }
  }
};

///////////////////////////////////////////////////////////////////////////////
// Download Event Handlers

function onUploadCancel(aEvent)
{
  var selectedIndex = gUploadsView.selectedIndex;

  gUploadManager.cancelUpload (aEvent.target.id);
  gUploadManager.canUpload ();

  setRDFProperty (aEvent.target.id, "DownloadAnimated", "false");

  gUploadViewController.onCommandUpdate ();

  // now reset the richlistbox
  gUploadsView.clearSelection ();
  var rowCount = gUploadsView.getRowCount ();
  if (selectedIndex >= rowCount)
    gUploadsView.selectedIndex = rowCount - 1;
  else
    gUploadsView.selectedIndex = selectedIndex;
}

function onDownloadPause(aEvent)
{
  var selectedIndex = gUploadsView.selectedIndex;

  var uri = aEvent.target.id;
  gUploadManager.pauseDownload(uri);
  setRDFProperty(uri, "DownloadStatus", aEvent.target.getAttribute("status-internal"));
  setRDFProperty(uri, "ProgressPercent", aEvent.target.getAttribute("progress"));

  // now reset the richlistbox
  gUploadsView.clearSelection();
  var rowCount = gUploadsView.getRowCount();
  if (selectedIndex >= rowCount)
    gUploadsView.selectedIndex = rowCount - 1;
  else
    gUploadsView.selectedIndex = selectedIndex;
}

function onDownloadResume(aEvent)
{
  var selectedIndex = gUploadsView.selectedIndex;

  gUploadManager.resumeDownload(aEvent.target.id);

  // now reset the richlistbox
  gUploadsView.clearSelection();
  var rowCount = gUploadsView.getRowCount();
  if (selectedIndex >= rowCount)
    gUploadsView.selectedIndex = rowCount - 1;
  else
    gUploadsView.selectedIndex = selectedIndex;
}

function onUploadRemove (aEvent)
{
  if (aEvent.target.removable)
    {
      var selectedIndex = gUploadsView.selectedIndex;
      gUploadManager.removeDownload (aEvent.target.id);
      gUploadViewController.onCommandUpdate ();
    }
}

function onDownloadShow(aEvent)
{
  var f = getLocalFileFromNativePathOrUrl(aEvent.target.id);

  if (f.exists()) {
    try {
      f.reveal();
    } catch (ex) {
      // if reveal failed for some reason (eg on unix it's not currently
      // implemented), send the file: URL window rooted at the parent to 
      // the OS handler for that protocol
      var parent = f.parent;
      if (parent) {
        openExternal(parent);
      }
    }
  }
  else {
    var brandStrings = document.getElementById("brandStrings");
    var appName = brandStrings.getString("brandShortName");
  
    var strings = document.getElementById("uploadStrings");
    var name = aEvent.target.getAttribute("target");
    var message = strings.getFormattedString("fileDoesNotExistError", [name, appName]);
    var title = strings.getFormattedString("fileDoesNotExistShowTitle", [name]);

    var promptSvc = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    promptSvc.alert(window, title, message);
  }
}

function onDownloadOpen(aEvent)
{
  if (aEvent.type == "dblclick" && aEvent.button != 0)
    return;
  var download = aEvent.target;
  if (download.localName == "download") {
    if (download.openable) {
      var f = getLocalFileFromNativePathOrUrl(aEvent.target.id);
      if (f.exists()) {
        if (f.isExecutable()) {
          var dontAsk = false;
          var pref = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefBranch);
          try {
            dontAsk = !pref.getBoolPref(PREF_BDM_ALERTONEXEOPEN);
          }
          catch (e) { }
          
          if (!dontAsk) {
            var strings = document.getElementById("uploadStrings");
            var name = aEvent.target.getAttribute("target");
            var message = strings.getFormattedString("fileExecutableSecurityWarning", [name, name]);

            var title = strings.getString("fileExecutableSecurityWarningTitle");
            var dontAsk = strings.getString("fileExecutableSecurityWarningDontAsk");

            var promptSvc = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
            var checkbox = { value: false };
            var open = promptSvc.confirmCheck(window, title, message, dontAsk, checkbox);
            
            if (!open) 
              return;
            else
              pref.setBoolPref(PREF_BDM_ALERTONEXEOPEN, !checkbox.value);              
          }        
        }
        try {
          f.launch();
        } catch (ex) {
          // if launch fails, try sending it through the system's external
          // file: URL handler
          openExternal(f);
        }
      }
      else {
        var brandStrings = document.getElementById("brandStrings");
        var appName = brandStrings.getString("brandShortName");
      
        var strings = document.getElementById("uploadStrings");
        var name = aEvent.target.getAttribute("target");
        var message = strings.getFormattedString("fileDoesNotExistError", [name, appName]);

        var title = strings.getFormattedString("fileDoesNotExistOpenTitle", [name]);

        var promptSvc = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        promptSvc.alert(window, title, message);
      }
    }
    else if(download.canceledOrFailed) {
      // If the user canceled this download, double clicking tries again. 
      fireEventForElement(download, "retry")
    }
  }
}

function onDownloadOpenWith(aEvent)
{
}

function onDownloadProperties(aEvent)
{
  window.openDialog("chrome://fa/content/downloads/downloadProperties.xul",
                    "_blank", "modal,centerscreen,chrome,resizable=no", aEvent.target.id);
}

function onDownloadAnimated(aEvent)
{
  gUploadViewController.onCommandUpdate();    

  setRDFProperty(aEvent.target.id, "DownloadAnimated", "true");
}

function onDownloadRetry(aEvent)
{
  var download = aEvent.target;
  if (download.localName == "download") {
    var src = getRDFProperty(download.id, "URL");
    var f = getLocalFileFromNativePathOrUrl(aEvent.target.id);
    saveURL (src, f, null, true, true);
  }

  gUploadViewController.onCommandUpdate();

  // retry always places the item in the first spot
  gUploadsView.selectedIndex = 0;
}

// This is called by the progress listener. We don't actually use the event
// system here to minimize time wastage. 
var gLastComputedMean = -1;
var gLastActiveDownloads = 0;
function onUpdateProgress()
{
  var numActiveDownloads = gActiveDownloads.length;
  if (numActiveDownloads == 0) {
    document.title = document.documentElement.getAttribute("statictitle");
    gLastComputedMean = -1;
    return;
  }
    
  var mean = 0;
  var base = 0;
  var dl = null;
  for (var i = 0; i < numActiveDownloads; ++i) {
    dl = gActiveDownloads[i];

    // gActiveDownloads is screwed so it's possible 
    // to have more files than we're really downloading.
    // The good news is that those files have size==0.
    // Same with files with unknown size. Their size==0.
    if (dl.percentComplete < 100 && dl.size > 0) {
      mean += dl.amountTransferred;
      base += dl.size;
    }
  }

  // we're not downloading anything at the moment,
  // but we already downloaded something.
  if (base == 0) {
    mean = 100;
  } else {
    mean = Math.floor((mean / base) * 100);
  }

  if (mean != gLastComputedMean || gLastActiveDownloads != numActiveDownloads) {
    gLastComputedMean = mean;
    gLastActiveDownloads = numActiveDownloads;
    
    var strings = document.getElementById("uploadStrings");
    
    var title;
    if (numActiveDownloads > 1)
      title = strings.getFormattedString("downloadsTitleMultiple", [mean, numActiveDownloads]);
    else
      title = strings.getFormattedString("downloadsTitle", [mean]);

    document.title = title;
  }
}

///////////////////////////////////////////////////////////////////////////////
// Startup, Shutdown
function Startup ()
{
  gUploadsView = document.getElementById ("uploadView");

  gUploadManager = new UploadManager ();
  
  // The UploadProgressListener (UploadProgressListener.js) handles progress
  // notifications. 
  var uploadStrings = document.getElementById("uploadStrings");
  gUploadListener = new UploadProgressListener (document, uploadStrings);
  gUploadManager.listener = gUploadListener;

  // The active downloads list is created by the front end only when the download starts,  
  // so we need to pre-populate it with any downloads that were already going. 
  /*
  var activeDownloads = gUploadManager.activeDownloads;
  var count = activeDownloads.Count ();
  for (var i = 0; i < count; ++i)
    gActiveDownloads.push (activeDownloads.QueryElementAt (i, Components.interfaces.nsIDownload));
  */

  // Handlers for events generated by the UI (downloads, list view)
  gUploadsView.addEventListener ("upload-cancel",        onUploadCancel,     false);

  //gUploadsView.addEventListener ("upload-pause",         onUploadPause,      false);
  //gUploadsView.addEventListener ("upload-resume",        onUploadResume,     false);
  gUploadsView.addEventListener ("upload-remove",        onUploadRemove,     false);
  //gUploadsView.addEventListener ("upload-show",          onUploadShow,       false);
  //gUploadsView.addEventListener ("upload-open",          onUploadOpen,       false);
  //gUploadsView.addEventListener ("upload-retry",         onUploadRetry,      false);
  //gUploadsView.addEventListener ("upload-animated",      onUploadAnimated,   false);
  //gUploadsView.addEventListener ("upload-properties",    onUploadProperties, false);
  //gUploadsView.addEventListener ("dblclick",             onUploadOpen,       false);

  // Set up AutoDownload display area
  //initAutoDownloadDisplay();
  //var pbi = Components.classes["@mozilla.org/preferences-service;1"]
  //                    .getService(Components.interfaces.nsIPrefBranch2);
  //pbi.addObserver ("browser.download.", gDownloadPrefObserver, false);

  // Handlers for events generated by the Download Manager (download events)
  var observerService = Components.classes[kObserverServiceProgID]
                                  .getService(Components.interfaces.nsIObserverService);
  observerService.addObserver (gUploadObserver, "up-done",   false);
  observerService.addObserver (gUploadObserver, "up-cancel", false);
  observerService.addObserver (gUploadObserver, "up-failed", false);  
  observerService.addObserver (gUploadObserver, "up-start",  false);  

  if ("arguments" in window)
    {
      if (window.arguments[0] == true)
	document.getElementById ("MsgLabel").value = "이제 업로드 관리자를 통해 업로드하세요.";
    }

  // This is for the "Clean Up" button, which requires there to be
  // non-active downloads before it can be enabled. 
  gUploadsView.controllers.appendController (gUploadViewController);

  // Finally, update the UI. 
  gUploadsView.database.AddDataSource (gUploadManager.datasource);
  gUploadsView.builder.rebuild ();
}

function Shutdown() 
{
  gUploadManager.listener = null;

  // Assert the current progress for all the downloads in case the window is reopened

  var observerService = Components.classes[kObserverServiceProgID]
                                  .getService(Components.interfaces.nsIObserverService);
  observerService.removeObserver(gUploadObserver, "up-done");
  observerService.removeObserver(gUploadObserver, "up-cancel");
  observerService.removeObserver(gUploadObserver, "up-failed");  
  observerService.removeObserver(gUploadObserver, "up-start");  
}

///////////////////////////////////////////////////////////////////////////////
// View Context Menus
var gContextMenus = [ 
  ["menuitem_pause", "menuitem_cancel", "menuseparator_properties", "menuitem_properties"],
  ["menuitem_open", "menuitem_show", "menuitem_remove", "menuseparator_properties", "menuitem_properties"],
  ["menuitem_retry", "menuitem_remove", "menuseparator_properties", "menuitem_properties"],
  ["menuitem_retry", "menuitem_remove", "menuseparator_properties", "menuitem_properties"],
  ["menuitem_resume", "menuitem_cancel", "menuseparator_properties", "menuitem_properties"]
];

function buildContextMenu (aEvent)
{
  if (aEvent.target.id != "downloadContextMenu")
    return false;
    
  var popup = document.getElementById("downloadContextMenu");
  while (popup.hasChildNodes())
    popup.removeChild(popup.firstChild);
  
  if (gUploadsView.selectedItem) {
    var idx = parseInt(gUploadsView.selectedItem.getAttribute("state"));
    if (idx < 0)
      idx = 0;
    
    var menus = gContextMenus[idx];
    for (var i = 0; i < menus.length; ++i)
      popup.appendChild(document.getElementById(menus[i]).cloneNode(true));
    
    return true;
  }
  
  return false;
}

///////////////////////////////////////////////////////////////////////////////
// Drag and Drop

var gUploadDNDObserver =
{
  onDragOver: function (aEvent, aFlavour, aDragSession)
  {
    aDragSession.canDrop = true;
  },
  
  onDrop: function(aEvent, aXferData, aDragSession)
  {
    if (aXferData.data != "") 
      {
	var f = aXferData.data;
	uploadFile (f);
      }
  },
  _flavourSet: null,  
  getSupportedFlavours: function ()
  {
    if (!this._flavourSet) {
      this._flavourSet = new FlavourSet();
      this._flavourSet.appendFlavour ("application/x-moz-file", "nsIFile");
    }
    return this._flavourSet;
  }
}

///////////////////////////////////////////////////////////////////////////////
// Command Updating and Command Handlers

var gUploadViewController = {
  supportsCommand: function (aCommand)
  {
    return aCommand == "cmd_cleanUp";
  },
  
  isCommandEnabled: function (aCommand)
  {
    if (aCommand == "cmd_cleanUp")
      return gUploadManager.canCleanUp;
    return false;
  },
  
  doCommand: function (aCommand)
  {
    if (aCommand == "cmd_cleanUp" && this.isCommandEnabled (aCommand)) {
      gUploadManager.cleanUp ();
      this.onCommandUpdate ();
    }
  },  
  
  onCommandUpdate: function ()
  {
    var command = "cmd_cleanUp";
    var enabled = this.isCommandEnabled (command);
    goSetCommandEnabled (command, enabled);
  }
};

function onDownloadShowOptions()
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var topWindow = windowManagerInterface.getMostRecentWindow("Browser:Options");
  if (topWindow) {
    topWindow.focus();
    topWindow.switchPage("catDownloadsButton");
  }
  else
    openDialog("chrome://browser/content/pref/pref.xul", "PrefWindow",
              "chrome,titlebar,resizable,modal", "catDownloadsButton");
}

function onDownloadShowInfo()
{
  if (gUploadsView.selectedItem)
    fireEventForElement(gUploadsView.selectedItem, "properties");
}

function initAutoDownloadDisplay ()
{
  var pref = Components.classes["@mozilla.org/preferences-service;1"]
                       .getService(Components.interfaces.nsIPrefBranch);

  var autodownload = pref.getBoolPref("browser.download.useDownloadDir");  

  if (autodownload) {
    var autodownloadInfo = document.getElementById("autodownloadInfo");
    autodownloadInfo.hidden = false;
    var autodownloadSpring = document.getElementById("autodownloadSpring");
    autodownloadSpring.hidden = true; 

    function getSpecialFolderKey (aFolderType) 
    {
//@line 737 "/Users/weongyo/Developments/fileauction/src/chrome/content/uploads/uploads.js"
      return aFolderType == "Desktop" ? "UsrDsk" : "UsrDocs";
//@line 745 "/Users/weongyo/Developments/fileauction/src/chrome/content/uploads/uploads.js"
      return "Home"; 
    }
    
    function getDownloadsFolder(aFolder)
    {
      var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"]
                                  .getService(Components.interfaces.nsIProperties);
      var dir = fileLocator.get(getSpecialFolderKey(aFolder), Components.interfaces.nsILocalFile);
      
      var bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
                              .getService(Components.interfaces.nsIStringBundleService);
      bundle = bundle.createBundle("chrome://fa/locale/downloads/unknownContentType.properties");

      var description = bundle.GetStringFromName("myDownloads");
      if (aFolder != "Desktop")
        dir.append(description);
        
      return dir;
    }

    var displayName = null;
    var folder;
    switch (pref.getIntPref("browser.download.folderList")) {
    case 0:
      folder = getDownloadsFolder("Desktop");
      displayName = document.getElementById("uploadStrings").getString("displayNameDesktop");
      break;
    case 1:
      folder = getDownloadsFolder("Downloads");
      break;
    case 2: 
      folder = pref.getComplexValue("browser.download.dir", Components.interfaces.nsILocalFile);
      break;
    }

    if (folder) {    
      var ioServ = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);
      var fph = ioServ.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
      var mozIconURI = "moz-icon://" +  fph.getURLSpecFromFile(folder) + "?size=16";
      var folderIcon = document.getElementById("saveToFolder")
      folderIcon.image = mozIconURI;
      
      var folderName = document.getElementById("saveToFolder");
      folderName.label = displayName || folder.leafName;
      folderName.setAttribute("path", folder.path);
    }
  }
  else {
    var autodownloadInfo = document.getElementById("autodownloadInfo");
    autodownloadInfo.hidden = true;
    var autodownloadSpring = document.getElementById("autodownloadSpring");
    autodownloadSpring.hidden = false; 
  }
}

var gDownloadPrefObserver = {
  observe: function (aSubject, aTopic, aData)
  {
    if (aTopic == "nsPref:changed") {
      switch(aData) {
      case "browser.download.folderList":
      case "browser.download.useDownloadDir":
      case "browser.download.dir":
        initAutoDownloadDisplay();
      }
    }
  }
};

function onDownloadShowFolder()
{
  var folderName = document.getElementById("saveToFolder");
  var dir = getLocalFileFromNativePathOrUrl(folderName.getAttribute("path"));
  if (!dir.exists())
   dir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0755);

  try {
    dir.reveal();
  } catch (ex) {
    // if nsILocalFile::Reveal failed (eg it currently just returns an
    // error on unix), just open the folder in a browser window
    openExternal(dir);
  }
}

function openExternal (aFile)
{
  var uri = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Components.interfaces.nsIIOService)
                      .newFileURI(aFile);

  var protocolSvc = 
      Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
                .getService(Components.interfaces.nsIExternalProtocolService);

  protocolSvc.loadUrl(uri);

  return;
}

// we should be using real URLs all the time, but until 
// bug 239948 is fully fixed, this will do...
function getLocalFileFromNativePathOrUrl(aPathOrUrl)
{
  if (aPathOrUrl.substring(0,7) == "file://") {

    // if this is a URL, get the file from that
    ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
      .getService(Components.interfaces.nsIIOService);

    // XXX it's possible that using a null char-set here is bad
    const fileUrl = ioSvc.newURI(aPathOrUrl, null, null).
      QueryInterface(Components.interfaces.nsIFileURL);
    return fileUrl.file.clone().
      QueryInterface(Components.interfaces.nsILocalFile);

  } else {
    // if it's a pathname, create the nsILocalFile directly
    var f = Components.classes["@mozilla.org/file/local;1"].
      createInstance(Components.interfaces.nsILocalFile);
    f.initWithPath(aPathOrUrl);

    return f;
  }
}

function sendSignalToCompany (query, status)
{
  var qstr = query + "&s=" + status;

  var req = new XMLHttpRequest ();
  req.open ("POST", URL_signal, false);
  req.setRequestHeader ('Content-Type', 'application/x-www-form-urlencoded');
  req.send (qstr);

  if (req.readyState == 4 && req.status == 200)
    {
      if (req.responseText.charAt (0) == "0")
	alert ("signal error");
    }
}

function addRDFUploadEntry (f, uri)
{
  var RDFContainer = '@mozilla.org/rdf/container;1';
  RDFContainer = Components.classes[RDFContainer].createInstance();
  RDFContainer = RDFContainer.QueryInterface(Components.interfaces.nsIRDFContainer);

  var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);

  RDFContainer.Init (gUploadManager.datasource, rdf.GetResource (NC_ROOT));

  var uriNode = rdf.GetResource (uri);

  if (RDFContainer.IndexOf (uriNode) >= 0)
    return;
  else
    {
      initUploadFile (f, uri);
      uriNode = rdf.GetResource (uri);
    }

  RDFContainer.AppendElement (uriNode);

  gUploadManager.saveState ();
}

function initUploadFile (f, uri)
{
  setRDFProperty (uri, "Name", f.leafName);
  setRDFProperty (uri, "UploadAnimated", "true");
  setRDFProperty (uri, "Transferred", "");
  setRDFProperty (uri, "DateStarted", "");
  setRDFProperty (uri, "DateEnded", "");
  setRDFProperty (uri, "ProgressPercent", "0");
  setRDFProperty (uri, "UploadState", "-1");
  setRDFProperty (uri, "File", uri);
  setRDFProperty (uri, "From", "From: " + prefs.getCharPref ("fileauction.login.userid"));
  setRDFProperty (uri, "To", "To: FileAuction");
}

function debugSocket (ex)
{
  dump (ex);
}

function errorSocket (ex)
{
  dump (ex);
}

function step1 (dataSocket, f)
{
  var host = "www.fatalbid.com";

  var last = '\r\n-----------------------------4713348823330\r\n';
  var data2 = '-----------------------------4713348823330\r\n'
    + 'Content-Disposition: form-data; name="fup"; filename="' + f.leafName +'"\r\n'
    + 'Content-Type: application/octet-stream\r\n\r\n';

  
  var data = "POST /fa/upload.php HTTP/1.1\r\n"
    + "Host: " + host + "\r\n"
    + "Keep-Alive: 300\r\n"
    + "Connection: keep-alive\r\n"
    + "Content-Type: multipart/form-data; boundary=---------------------------4713348823330\r\n"
    + "Content-Length: " + (data2.length + f.fileSize + last.length) + "\r\n\r\n"
    + data2;

  setRDFProperty (dataSocket.uri, "UploadState", "0");

  return dataSocket.dataOutstream.write (data, data.length);
}

function step2 (dataSocket)
{
  var last = '\r\n-----------------------------4713348823330\r\n';

  return dataSocket.dataOutstream.write (last, last.length);
}

function step3 (tm)
{
  tm.onInputRead = true;

  //////////////////////////////////////////////
  var dataListener = 
    {
      data : "",
      onStartRequest: function (request, context) { },
      onStopRequest: function (request, context, status) 
      { 
        tm.onInputRead = false;
        tm.parent.kill ();
      },
      onDataAvailable: function (request, context, inputStream, offset, count)
      {
	this.data += tm.dataInstream.read (50, count);
	if (this.data.lastIndexOf ('FileAuction_1') != -1)
          {
            gUploadManager.canUpload ();
            setRDFProperty (tm.uri, "UploadState", "1");
            tm.onInputRead = false;
          }
	else if (this.data.lastIndexOf ('FileAuction_0') != -1)
          {
            gUploadManager.canUpload ();
            setRDFProperty (tm.uri, "UploadState", "2");
            tm.onInputRead = false;
          }
      }
    };

  var pump = Components.classes["@mozilla.org/network/input-stream-pump;1"]
	               .createInstance(Components.interfaces.nsIInputStreamPump);
  pump.init (tm.inputSocket, -1, -1, 0, 0, false);
  pump.asyncRead (dataListener, null);
}

function updateProgressBar (tm)
{
  var percent = (100 * tm.bytesUploaded) / tm.bytesTotal;

  setRDFProperty (tm.uri, "ProgressPercent", "" + percent);
}

function uploadFile (f)
{
  if (!checkLoginStatus ())
    {
      document.getElementById ("MsgLabel").value = '파일 업로드를 위해서는 먼저 로그인을 해야 합니다.';
      return;
    }

  if (gUploadManager.activeUploadsCount > 0)
    {
      document.getElementById ("MsgLabel").value = '동시에 1 개의 파일 업로드만 가능합니다.';
      return;
    }
  document.getElementById ("MsgLabel").value = '';
  gUploadManager.activeUploadsCount++;

  var uri = getFileURIFromLocalPath (f.path);

  addRDFUploadEntry (f, uri);
  
  try 
    {
      var proxy        = { proxyType: "", proxyHost: "", proxyPort: -1 };
      
      var dataSocket = new FileAuctionDataSocketMozilla (false, proxy, "fbay.maru.net", 80);
      dataSocket.debug = debugSocket;
      dataSocket.error = errorSocket;
      dataSocket.uri   = uri;
      dataSocket.step1 = step1;
      dataSocket.step2 = step2;
      dataSocket.step3 = step3;
      dataSocket.updateProgressBar = updateProgressBar;
      gUploadManager.addUpload (uri, dataSocket);
      dataSocket.connect (true, f.path, 0, 0);
    }
  catch (e)
    {
    }
  document.getElementById ("MsgLabel").value = "업로드중입니다...";
}

function getPLocalFile (filename)
{
  try 
    {
      netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
  catch (e) 
    {
      alert ("Permission to read file was denied.");
    }

  var file = Components.classes["@mozilla.org/file/local;1"]
   		       .createInstance(Components.interfaces.nsILocalFile);
  file.initWithPath (filename);

  fstream = Components.classes["@mozilla.org/network/file-input-stream;1"]
		     .createInstance(Components.interfaces.nsIFileInputStream);
  fstream.init (file, 0x01, 00004, null);

  return fstream;
}
